/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef UBLAS_PLUS_H
#define UBLAS_PLUS_H

#include "maxill.h"

/*
begin_html
See also:
<ul>
<li><a href="ublas_matrixutil.h.html">ublas_matrixutil.h</a></li>  
</ul>
<b>Up:</b> <a href="index.html">Index</a>
end_html
*/

// This header includes the vector and matrix from boost::ublas,
// and defines template functions to set up, read, print, and store them.

#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>

using namespace boost::numeric::ublas;

template<class M, class T>
  void matrix_from_arrays(M& mat, T** dat, int m, int n)
{
  if ( mat.size1() != m || mat.size2() != n )
    mat.resize(m, n);
  // typename M::iterator1; // so the compiler doesn't treat it as R-value
  typename M::iterator1 mat_i = mat.begin1();
  for (int i=0; i<m; ++i, ++mat_i) {
#ifndef BOOST_UBLAS_NO_NESTED_CLASS_RELATION
    typename M::iterator2 mat_j = mat_i.begin();
#else
    typename M::iterator2 mat_j = begin(mat_i, iterator1_tag());
#endif
    T* dat_j = dat[i];
    for (int j=0; j<n; ++j)
      *(mat_j++) = *(dat_j++);
  }
}

template<class M, class T>
  void matrix_to_arrays(M& mat, T** dat, int m, int n)
{
  typename M::iterator1 mat_i = mat.begin1();
  for (int i=0; i<m; ++i, ++mat_i) {
#ifndef BOOST_UBLAS_NO_NESTED_CLASS_RELATION
    typename M::iterator2 mat_j = mat_i.begin();
#else
    typename M::iterator2 mat_j = begin(mat_i, iterator1_tag());
#endif
    T* dat_j = dat[i];
    for (int j=0; j<n; ++j)
      *(dat_j++) = *(mat_j++);
  }
}

template<class M, class T>
  void matrix_from_array(M& mat, T* dat, int m, int n)
{
  if ( mat.size1() != m || mat.size2() != n )
    mat.resize(m, n);
  // typename M::iterator1; // so the compiler doesn't treat it as R-value
  typename M::iterator1 mat_i = mat.begin1();
  T* dat_i = dat;
  for (int i=0; i<m; ++i, ++mat_i) {
#ifndef BOOST_UBLAS_NO_NESTED_CLASS_RELATION
    typename M::iterator2 mat_j = mat_i.begin();
#else
    typename M::iterator2 mat_j = begin(mat_i, iterator1_tag());
#endif
    for (int j=0; j<n; ++j) {
      *(mat_j++) = *(dat_i++);
    }
  }
}

template<class M, class T>
  void matrix_to_array(M& mat, T* dat, int m, int n)
{
  if ( mat.size1() != m || mat.size2() != n )
    mat.resize(m, n);
  // typename M::iterator1; // so the compiler doesn't treat it as R-value
  typename M::iterator1 mat_i = mat.begin1();
  T* dat_i = dat;
  for (int i=0; i<m; ++i, ++mat_i) {
#ifndef BOOST_UBLAS_NO_NESTED_CLASS_RELATION
    typename M::iterator2 mat_j = mat_i.begin();
#else
    typename M::iterator2 mat_j = begin(mat_i, iterator1_tag());
#endif
    for (int j=0; j<n; ++j) {
      *(dat_i++) = *(mat_j++);
    }
  }
}

template<class T>
void matrix_to_partition(matrix<T>& mat, int m, int n, int *rowix, int *colix, matrix<T>& part)
{
  if ( part.size1() != m || part.size2() != n )
    part.resize(m, n);
  for (int i=0; i<m; ++i)
    for (int j=0; j<n; ++j)
      part(i,j) = mat(rowix[i], colix[j]);
}

template<class T>
void matrix_from_partition(matrix<T>& mat, int m, int n, int *rowix, int *colix, matrix<T>& part)
{
  if ( part.size1() != m || part.size2() != n )
    part.resize(m, n);
  for (int i=0; i<m; ++i)
    for (int j=0; j<n; ++j)
      mat(rowix[i], colix[j]) = part(i,j);
}

template<class T>
void fill_identity(matrix<T>& mat)
{
  int m = mat.size1();
  int n = mat.size2();
  for (int i=0; i<m; ++i)
    for (int j=0; j<n; ++j)
      mat(i,j) = (i==j) ? 1 : 0;
}

template<class T>
void fill_const(matrix<T>& mat, T val)
{
  for (typename matrix<T>::iterator1 ii = mat.begin1(); ii != mat.end1(); ++ii)
#ifndef BOOST_UBLAS_NO_NESTED_CLASS_RELATION
    for (typename matrix<T>::iterator2 jj = ii.begin(); jj != ii.end(); ++jj)
#else
    for (typename matrix<T>::iterator2 jj = begin(ii, iterator1_tag()); jj != end(ii, iterator1_tag()); ++jj)
#endif
      *jj = val;
}

template<class T>
std::ostream& operator<<(std::ostream& ost, vector<T>& vec)
{
  ost << "[ ";
  for (typename vector<T>::iterator ii = vec.begin(); ii != vec.end(); ++ii)
    ost << *ii << ' ';
  ost << "]";
  return ost;
}

template<class T>
std::ostream& operator<<(std::ostream& ost, matrix<T>& mat)
{
  ost << "[ ";
  for (typename matrix<T>::iterator1 ii = mat.begin1(); ii != mat.end1(); ++ii) {
    ost << "[ ";
#ifndef BOOST_UBLAS_NO_NESTED_CLASS_RELATION
    for (typename matrix<T>::iterator2 jj = ii.begin(); jj != ii.end(); ++jj) {
#else
    for (typename matrix<T>::iterator2 jj = begin(ii, iterator1_tag()); jj != end(ii, iterator1_tag()); ++jj) {
#endif
      ost << *jj << ' ';
    }
    ost << "]" << std::endl << "  ";
  }
  ost << "]";
  return ost;
}

  // These are to simplify working with C-interfaced code, which variously expects T* or T**
  // e.g. in max_mac_ll.cpp

template<class T>
class pointy_matrix {
public:
  matrix<T> m;
  std::vector<T*> rows;
  T *p, **pp;
  int n; // == m.size1() * m.size2()
  
  pointy_matrix()
    : p(NULL), pp(NULL), n(0)
  {}
  pointy_matrix(int nr, int nc)
    : m(nr, nc), rows(nr), p(NULL), pp(NULL), n(nr*nc)
  {
    resize(nr, nc);
  }
  void resize(int nr, int nc)
  {
    n = nr*nc;
    m.resize(nr, nc);
    m.clear();
    rows.resize(nr);
    for (int i=0; i<nr; ++i)
      rows[i] = &(m(i,0));
    if ( nr && nc ) {
      p = &(m(0,0));
      pp = &(rows[0]);
    }
  }
  void clear()
  {
    m.clear();
  }
};

template<class T>
class pointy_col_vector {
public:
  matrix<T> m;
  T *p, **pp;
  int n;

  pointy_col_vector()
    : p(NULL), pp(&p), n(0)
  {}
  pointy_col_vector(int n)
    : m(n,1), p(NULL), pp(&p), n(n)
  {
    resize(n);
  }
  pointy_col_vector(int n, T *data)
    : m(n,1), p(NULL), pp(&p), n(n)
  {
    resize(n);
    memcpy(p, data, n*sizeof(T));
  }
  void resize(int n)
  {
    this->n = n;
    m.resize(n,1);
    m.clear();
    if ( n )
      p = &(m(0,0));
  }
  void clear()
  {
    m.clear();
  }
};
    

#ifdef _WIN32
template class MAXILL_API boost::numeric::ublas::vector<char>;
template class MAXILL_API boost::numeric::ublas::matrix<char>;
template class MAXILL_API boost::numeric::ublas::vector<short>;
template class MAXILL_API boost::numeric::ublas::matrix<short>;
template class MAXILL_API boost::numeric::ublas::vector<int>;
template class MAXILL_API boost::numeric::ublas::matrix<int>;
template class MAXILL_API boost::numeric::ublas::vector<float>;
template class MAXILL_API boost::numeric::ublas::matrix<float>;
template class MAXILL_API boost::numeric::ublas::vector<double>;
template class MAXILL_API boost::numeric::ublas::matrix<double>;
#endif

  
  
#endif
